# Copyright 2020-2023 Leibniz Information Centre for Economics
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


def uncase_first_char(text: str) -> str:
    if text[0].isalpha():
        return "({}|{}){}".format(
            text[0].upper(),
            text[0].lower(),
            text[1:]
            )
    else:
        return text


def sentence_case_handler(text: str) -> str:
    if text.isupper():
        return text
    return uncase_first_char(text)


def title_case_handler(text: str) -> str:
    if text.isupper():
        return text
    return ' '.join(map(uncase_first_char, text.split()))
