# Copyright 2020-2023 Leibniz Information Centre for Economics
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import stwfsapy.thesaurus as t


def test_label_unwrap(tuples):
    unwrapped = list(t._unwrap_labels(tuples))
    assert len(tuples) == len(unwrapped)
    for original, transformed in zip(tuples, unwrapped):
        assert isinstance(transformed[1], str)
        assert original[1].toPython() == transformed[1]
        assert original[0] == transformed[0]
